// ------------------------------------------------------------------------
// rminode2.c
// 	2005 by HellOnNET.TK
// 	http://www.hellonnet.tk
// ------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>

#include <mntent.h>				// mtab

// Se requieren las e2fslibs-dev !!! :
#include <ext2fs/ext2fs.h>

// Necesari per la funcio get_current_dir_name:
#ifndef __USE_GNU
 #define __USE_GNU
#endif

#define MAX_MOUNTED_FILESYS 20

typedef enum { true=0, false} bool;

// Prototipos de funciones :
void perr(char *, int, bool);
void getPhisDevice (char * , char *);
int stab_cmp (void *, void *);

int main (int argc, char ** argv) {

	char * curDir, phisDevice[256];
	DIR * dir;
	struct dirent * dirEntry;
	int inode;

	ext2_filsys fSys;
	ext2_ino_t i_curInode, i_curDir;

	printf("rminode by HellOnNET.TK\n");

	//
	// Obtenir llista de fitxers del directori actual i els seus
	// i-nodes
	//
	// NOTA: Los archivos regulares son los unicos que pueden 
	// borrarse via rminode. El resto se muestra como ( ! )
	//
	if(NULL==(curDir = get_current_dir_name()))
		perr("get_current_dir_name",errno,true);
	if(NULL==(dir = opendir(curDir)))perr("open_dir",errno,true);

	dirEntry = readdir(dir);
	while(dirEntry!=NULL){
		printf("%s [I: %d] (%s)\n", dirEntry->d_name, dirEntry->d_ino,
				(dirEntry->d_type==DT_REG)?"REGULAR":"!");
		// El "." es el i-node del directorio actual :
		if(strcmp(".",dirEntry->d_name)==0)i_curDir=dirEntry->d_ino;
		dirEntry = readdir(dir);
	}
	closedir(dir);

	// Obtener el dispositivo fisico para el directorio actual :
	getPhisDevice(curDir,phisDevice);
	if(phisDevice==NULL) perr("getPhisDevice",-1,false);

	//
	// Solicitar un i-node al usuari per esborrar :
	//
	printf("Indica el i-node a BORRAR dins de %s: ", phisDevice);
	scanf("%d", &i_curInode);

	// Abrir el sistema de ficheros para trabajar con ext2fs :
	if(ext2fs_open(phisDevice,EXT2_FLAG_RW,0,0,unix_io_manager,&fSys)!=0)
		perr("ext2fs_open",errno,true);
	if(ext2fs_unlink(fSys,i_curDir,NULL,i_curInode,0)!=0)
		perror("ext2fs_unlink");
	else printf("I-node %d en dir %d en device %s ELIMINADO.\n",
			i_curInode, i_curDir, phisDevice);

	// Sincronizar sistema de ficheros y cerrar :
	ext2fs_flush(fSys);
	ext2fs_close(fSys);
	sync();

	printf("Puede ser necesario DESMONTAR y REMONTAR %s...\n\n",phisDevice);
	printf("--------------------------\n");
	printf("http://www.hellonnet.tk\n");
	printf("--------------------------\n");

	return 0;
}

//
// Mostrar error por pantalla y salir retornando errno :
//
void perr(char * message, int retval, bool errSys) {
	if(errSys==true)perror(message);
	else printf("%s\n",message);
	exit(retval);
}

//
// Obtenir el device a partir del directori actual
//
void getPhisDevice (char * dir, char * buffer) {

	FILE * mtab;
	struct mntent * mtabEntry;
	char stab[MAX_MOUNTED_FILESYS][256], 
	     	 stab_device[MAX_MOUNTED_FILESYS][256];
	int istab=0, _istab;

	void * vfunc = stab_cmp;

	if(NULL==(mtab = setmntent("/etc/mtab","r"))){
		buffer = NULL; return;
	}
	
	// Obtener coindidencias mas similares en "prefix-dir" :
	mtabEntry = getmntent(mtab);
	while(mtabEntry!=NULL){
		if(strstr(dir,mtabEntry->mnt_dir)!=NULL){
			if(istab>=MAX_MOUNTED_FILESYS) break;
			strcpy(stab[istab],mtabEntry->mnt_dir);
			strcpy(stab_device[istab],mtabEntry->mnt_fsname);
			istab++;
		} mtabEntry = getmntent(mtab);
	} endmntent(mtab);

	// La entrada + larga coincidente sera la valida (la ultima tras
	// qsort) :
	qsort(stab,istab,256,vfunc);
	strcpy(buffer,stab_device[istab-1]);

}

int stab_cmp (void * ar1, void * ar2) {
	return strcmp(ar1,ar2);
}
